;; INITIAL DICTIONARY
	Nforth_dict	string "forth_dict"

%macro entry 3	;; entry <address>, <name>, <flags>
	dd	%1		;address
	dd	e%1 - %1	;size
	dd	%3 | T_FORTH	;flags
	dd	%%ne-%%n-1	;name length
%%n	string %2		;name
%%ne
%endmacro

table:
;;		label		name		flags
;;
	;; Text/number I/O
	entry	BYE, 		"bye",		0
	entry	PRINT,		".",		0
;	entry	UPRINT,		"u.",		0
	entry	PRINT_NUM_PAD,	"p.",		0
	entry	BASE,		"base",		0
	entry	TYPE,		"type",		0
	entry	EMIT,		"emit",		0
	entry	ACCEPT,		"accept",	0
	entry	KEY,		"key",		0
;	entry	EKEY,		"ekey",		0
;	entry	KEY?,		"key?",		0
	entry   CR,		"cr",		0

	;; Low-level					Assembler equivalent:
	entry	_INT,		"int",		0	; INT n
	entry	IOR,		"ior",		0	; IN byte
	entry	IOW,		"iow",		0	; OUT
	entry	IOR,		"inb",		0	; IN byte
	entry	IOW,		"outb",		0	; OUT
;	entry	INW,		"inw",		0	; IN word
;	entry	OUTW,		"outw",		0	; OUT
;	entry	IND,		"ind",		0	; IN dword
;	entry	OUTD,		"outd",		0	; OUT

	;; Compilation
	entry	COMMENT,	"\",		F_IMM
	entry	PARSE,		"parse",	0
	entry	PARSE_WORD,	"parse-word",	0
	entry	LITERAL,	"literal",	F_IMM | F_COM
	entry	LEFT_BRACKET,	"[",		F_IMM
	entry	RIGHT_BRACKET,	"]",		0
	entry	START_COMPILE,	":",		0
	entry	END_COMPILE, 	";",		F_IMM | F_COM
	entry	CONSTANT,	"constant",	0
	entry	VARIABLE,	"variable",	0
	entry	IMMEDIATE,	"immediate",	0
	entry	COMPILE_ONLY,	"compile-only",	0
	entry	POSTPONE,	"postpone",	F_IMM | F_COM
	entry	CREATE,		"create",	0		;finish!
	entry	DOES,		"does>",	0		;finish!
	entry	TICK,		"'",		0
	entry	EXECUTE,	"execute",	0
	entry	COMPILE,	"[compile]",	F_IMM

	;; Memory access
	entry	FETCH,		"@",		0
	entry	STORE,		"!",		0
	entry	PLUS_STORE,	"+!",		0
	entry	C_FETCH,	"c@",		0
	entry	C_STORE,	"c!",		0
	entry	W_FETCH,	"w@",		0
	entry	W_STORE,	"w!",		0
	entry	ALIGNED,	"aligned",	0
	entry	_ALIGN,		"align",	0
	entry	HERE,		"here",		0
	entry	COMMA,		",",		0
	entry	C_COMMA,	"c,",		0
	entry	_CMOVE,		"cmove",	0
	entry	_CMOVE2,	"cmove>",	0
	entry	FILL,		"fill",		0
	entry	ALLOT,		"allot",	0
	entry	_1PLUS,		"char+",	0		;alias to 1+
	entry	CELL_PLUS,	"cell+",	0

	;; Stack
	entry	DUP,		"dup",		0
	entry	?DUP,		"?dup",		0
	entry	_2DUP,		"2dup",		0
	entry	_DROP,		"drop",		0
	entry	_2DROP,		"2drop",	0
	entry	_2SWAP,		"2swap",	0
	entry	_2OVER,		"2over",	0
	entry	SWAP,		"swap",		0
	entry	OVER,		"over",		0
	entry	ROT,		"rot",		0
	entry	_ROT,		"-rot",		0
	entry	ROLL,		"roll",		0
	entry	DEPTH,		"depth",	0
	entry	TRACE,		".s",		0
	entry	R_FROM,		"r>",		F_COM | F_IMM
	entry	R_FETCH,	"r@",		F_COM | F_IMM
	entry	TO_R,		">r",		F_COM | F_IMM
	entry	PICK,		"pick",		0

	;; Math
	entry	ABS,		"abs",		0
	entry	MIN,		"min",		0
	entry	MAX,		"max",		0
	entry	NEGATE,		"negate",	0
	entry	PLUS,		"+",		0
	entry	MINUS,		"-",		0
	entry	_1PLUS,		"1+",		0
	entry	_1MINUS,	"1-",		0
	entry	_2MUL,		"2*",		0
	entry	_2DIV,		"2/",		0
	entry	MULTIPLY,	"*",		0
	entry	DIVIDE,		"/",		0
	entry	MUL_DIV,	"*/",		0
	entry	MOD,		"mod",		0
	entry	DIV_MOD,	"/mod",		0
	entry	MUL_DIV_MOD,	"*/mod",	0
	entry	M_MULTIPLY,	"m*",		0
	entry	UM_MOD,		"um/mod",	0

	;; Logic
	entry   BINAND,         "and",          0
	entry   BINOR,          "or",           0
	entry   BINXOR,         "xor",          0
	entry	INVERT,		"invert",	0
	entry	LSHIFT,		"lshift",	0
	entry	RSHIFT,		"rshift",	0

	;; Conditionals
	entry   EQUAL?,         "=",            0
	entry   LESS?,          "<",            0
	entry   ULESS?,         "u<",           0 ; "U<" would look better
	entry   GREATER?,       ">",            0

	;; Branching
	entry	IF,		"if",		F_IMM | F_COM
	entry	ELSE,		"else",		F_IMM | F_COM
	entry	THEN,		"then",		F_IMM | F_COM
;	entry	CASE,		"case",		F_IMM | F_COM
;	entry	ENDCASE,	"endcase",	F_IMM | F_COM
;	entry	OF,		"of",		F_IMM | F_COM
;	entry	ENDOF,		"endof",	F_IMM | F_COM

	;; Looping
	entry	BEGIN,		"begin",	F_IMM | F_COM
	entry	WHILE,		"while",	F_IMM | F_COM
	entry	REPEAT,		"repeat",	F_IMM | F_COM
	entry	UNTIL,		"until",	F_IMM | F_COM
	entry	AGAIN,		"again",	F_IMM | F_COM
	entry	DO,		"do",		F_IMM | F_COM
	entry	_LOOP,		"loop",		F_IMM | F_COM
	entry	PLOOP,		"+loop",	F_IMM | F_COM
	entry	I,		"i",		F_COM
	entry	J,		"j",		F_COM
;	entry	_LEAVE,		"leave",	F_IMM | F_COM
;	entry	UNLOOP,		"unloop",	F_IMM | F_COM

	;; Extras
	entry	MD,		"md",		0	;Memory Dump
	entry	PAGE,		"page",		0	;Clear screen
	entry	AT_XY,		"at-xy",	0	;Move cursor
	entry	FONT,		"font",		0	;font table ptr
	entry	GPUTC,		"gputc",	0
	entry	DELAY,		"delay",	0	;port 61h delay

	;; Hard disk
	entry	HDR,		"hdr",		0	;Hard Disk Read
	entry	HDW,		"hdw",		0	;Hard Disk Write

	;; Floppy disk
	entry	CHS_convert,	"lba>chs",	0
	entry	FDR,		"fdr",		0	;Floppy Disk Read
	entry	read_sector,	"rs",		0	;temporary--testing

	;; "Higher-level disk & filesystem stuff"
	entry	INIT_FAT,	"init-fat",	0
	entry	SECTOR,		"sector",	0
	entry	PARSE_FN,	"parse-fn",	0
	entry	SEARCH_FN,	"search-fn",	0
	entry	LOAD_FILE,	"load-file",	0

	;; Serial
;	entry INIT_COM,		"init-com",	0 \Now called in pmode.inc
	entry SET_DTR,		"set-dtr",	0
	entry RESET_DTR,	"reset-dtr",	0        
	entry SET_RTS,		"set-rts",	0
	entry RESET_RTS,	"reset-rts",	0
	entry SEND_BREAK,	"send-break",	0
	entry WRITE_COM_BYTE,	"write-com-byte",	0
	entry READ_COM_BYTE,	"read-com-byte",	0
	entry SET_COM_SPEED,	"set-com-speed",	0
	entry SET_COM_BITS,	"set-com-bits", 0
	entry SEND_BREAK,	"send-break",	0	

	;; Graphics
	entry	HLINE,		"hline",	0
	entry	VLINE,		"vline",	0
	entry	PUTPIXEL,	"putpixel",	0
	entry	BLIT,		"blit",		0

	dd	0
