/* cwrapper.c - C wrapper for Retro/Tunes

  Copyright (c)1999 by Thomas Novelli

  This software is distributed under the terms of the GNU General Public
  License (included in the file LICENSE)

Contains:
  Startup code
  I/O wrappers
  Interactive terminal I/O initializations, borrowed from PFE
*/


#include <stdio.h>
#include <termios.h>

#ifndef STDIN_FILENO
# define STDIN_FILENO 0
#endif

#define C_IFLAGS_OFF    (ICRNL | IGNBRK | IGNCR | INLCR | \
                         ISTRIP | IXOFF | IXON)
#define C_IFLAGS_ON     (BRKINT)
#define C_OFLAGS_OFF    (0)
#define C_OFLAGS_ON     (0)
#define C_LFLAGS_OFF    (ECHO | ICANON)
#define C_LFLAGS_ON     (ISIG)


static struct termios tty_system;
#define tty_save()      tcgetattr (STDIN_FILENO, &tty_system)
#define tty_restore()   tcsetattr (STDIN_FILENO, TCSAFLUSH, &tty_system)
#ifdef CBAUD
# define tty_ospeed     (tty_system.c_cflag & CBAUD)
#else
# define tty_ospeed     (tty_system.c_ospeed)
#endif

static void
tty_interactive (void)
{
  struct termios tty = tty_system;
  				// Note: option.canonical is a PFE option
  if (!isatty (STDIN_FILENO) /* || option.canonical */)
    return;
  tty.c_iflag &= ~C_IFLAGS_OFF, tty.c_iflag |= C_IFLAGS_ON;
  tty.c_oflag &= ~C_OFLAGS_OFF, tty.c_oflag |= C_OFLAGS_ON;
  tty.c_lflag &= ~C_LFLAGS_OFF, tty.c_lflag |= C_LFLAGS_ON;
  tty.c_cc[VMIN] = 1;
  tty.c_cc[VTIME] = 0;
//  tty.c_cc[VINTR] = INTR_KEY;
//  tty.c_cc[VQUIT] = QUIT_KEY;
#if defined HAVE_TERMIOS_H
  tty.c_cc[VSUSP] = SUSP_KEY;
  tty.c_cc[VSTART] = '\xFF';
  tty.c_cc[VSTOP] = '\xFF';
#endif
  tcsetattr (STDIN_FILENO, TCSAFLUSH, &tty);
}
                               


void main ()
{
	printf(">> Retro starting.\n\n");

	// Initialize terminal for interactive I/O
	tty_save();
	tty_interactive();

        start(69);

	// Restore previous terminal state
	tty_restore();

        printf("\n>> Retro exited cleanly.\n");
}


